@@ -51,6 +51,14 @@ typedef struct
     gsize data_length;
 } ScanLine;
 
+typedef enum
+{
+    SCAN_MODE_DEFAULT,    
+    SCAN_MODE_COLOR,
+    SCAN_MODE_GRAY,
+    SCAN_MODE_LINEART
+} ScanMode;
+
 
 typedef struct ScannerPrivate ScannerPrivate;
 
@@ -64,21 +72,28 @@ typedef struct
 {
     GObjectClass parent_class;
 
-    void (*ready) (Scanner *scanner);
     void (*update_devices) (Scanner *scanner, GList *devices);
+    void (*authorize) (Scanner *scanner, const gchar *resource);
     void (*got_page_info) (Scanner *scanner, ScanPageInfo *info);
     void (*got_line) (Scanner *scanner, ScanLine *line);
     void (*scan_failed) (Scanner *scanner, GError *error);
-    void (*image_done) (Scanner *scanner);
+    void (*page_done) (Scanner *scanner);
+    void (*document_done) (Scanner *scanner);
 } ScannerClass;
 
 
-Scanner *scanner_new ();
+GType scanner_get_type (void);
+
+Scanner *scanner_new (void);
 
 void scanner_start (Scanner *scanner);
 
-void scanner_scan (Scanner *scanner, const char *device, const char *source,
-                   gint dpi, const char *scan_mode, gint depth, gboolean multi_page);
+void scanner_authorize (Scanner *scanner, const gchar *username, const gchar *password);
+
+void scanner_redetect (Scanner *scanner);
+
+void scanner_scan (Scanner *scanner, const char *device,
+                   gint dpi, ScanMode scan_mode, gint depth, gboolean multi_page);
 
 void scanner_cancel (Scanner *scanner);
 
