@@ -54,7 +54,8 @@ struct SimpleScanPrivate
     Book *book;
     BookView *book_view;
     gboolean updating_page_menu;
-    Orientation default_orientation;
+    gint default_page_width, default_page_height, default_page_dpi;
+    Orientation default_page_orientation;
     gboolean book_is_placeholder; // FIXME: Needs to be cleared when scan starts
 
     gchar *default_file_name;
@@ -131,38 +132,56 @@ ui_authorize (SimpleScan *ui, const gchar *resource, gchar **username, gchar **p
     *password = g_strdup (gtk_entry_get_text (GTK_ENTRY (ui->priv->password_entry)));
 }
 
-
 void
-ui_mark_devices_undetected (SimpleScan *ui)
+ui_set_scan_devices (SimpleScan *ui, GList *devices)
 {
     GtkTreeIter iter;
-    
-    if (gtk_tree_model_get_iter_first (ui->priv->device_model, &iter)) {
-        do {
-            gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 2, FALSE, -1);
-        } while (gtk_tree_model_iter_next (ui->priv->device_model, &iter));
+    GList *i;
+    gboolean have_iter;
+    gboolean have_selection;
+  
+    have_selection = gtk_combo_box_get_active (GTK_COMBO_BOX (ui->priv->device_combo)) >= 0;
+  
+    /* Remove disappeared devices */
+
+    do {
+        for (have_iter = gtk_tree_model_get_iter_first (ui->priv->device_model, &iter);
+             have_iter;
+             have_iter = gtk_tree_model_iter_next (ui->priv->device_model, &iter)) {
+            gchar *name;
+
+            gtk_tree_model_get (ui->priv->device_model, &iter, 0, &name, -1);
+            for (i = devices; i; i = i->next)
+                if (strcmp (name, ((ScanDevice *) i->data)->name) == 0)
+                    break;
+            g_free (name);
+
+            /* Device was removed */
+            if (i == NULL) {
+                gtk_list_store_remove (GTK_LIST_STORE (ui->priv->device_model), &iter);
+                break;
+            }
+        }
+    } while (have_iter);
+
+    /* Add new devices */
+    for (i = devices; i; i = i->next) {
+        ScanDevice *device = (ScanDevice *) i->data;
+
+        if (!find_scan_device (ui, device->name, &iter)) {
+            gtk_list_store_append (GTK_LIST_STORE (ui->priv->device_model), &iter);
+            gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 0, device->name, 1, device->label, -1);
+
+            /* Select this device if none selected */
+            if (!have_selection) {
+                gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter);
+                have_selection = TRUE;
+            }
+        }
     }
 }
 
 
-void
-ui_add_scan_device (SimpleScan *ui, const gchar *device, const gchar *label)
-{
-    GtkTreeIter iter;
-    
-    if (!find_scan_device (ui, device, &iter)) {
-        gtk_list_store_append (GTK_LIST_STORE (ui->priv->device_model), &iter);
-        gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 0, device, -1);
-    }
-
-    gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 1, label, 2, TRUE, -1);
-    
-    /* Select this device if none selected */
-    if (gtk_combo_box_get_active (GTK_COMBO_BOX (ui->priv->device_combo)) == -1)
-        gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter);
-}
-
-
 gchar *
 ui_get_selected_device (SimpleScan *ui)
 {
@@ -199,9 +218,10 @@ add_default_page (SimpleScan *ui)
     if (book_get_n_pages (ui->priv->book) > 0)
         return;
 
-    /* Start with A4 white image at 72dpi */
-    // FIXME: Remember last page dimensions
-    book_append_page (ui->priv->book, 595, 842, 72, ui->priv->default_orientation);
+    book_append_page (ui->priv->book,
+                      ui->priv->default_page_width, ui->priv->default_page_height,
+                      ui->priv->default_page_dpi,
+                      ui->priv->default_page_orientation);
 
     /* Remove this page on the next scan */
     ui->priv->book_is_placeholder = TRUE;
@@ -269,13 +289,11 @@ scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     gchar *device, *mode;
 
     device = ui_get_selected_device (ui);
-    if (device) {
-        mode = get_document_hint (ui);
-        g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
-                       FALSE, get_replace_pages (ui));
-        g_free (device);
-        g_free (mode);
-    }
+    mode = get_document_hint (ui);
+    g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
+                   FALSE, get_replace_pages (ui));
+    g_free (device);
+    g_free (mode);
 }
 
 
@@ -299,13 +317,11 @@ continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         gchar *device, *mode;
 
         device = ui_get_selected_device (ui);
-        if (device) {
-            mode = get_document_hint (ui);
-            g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
-                           TRUE, get_replace_pages (ui));
-            g_free (device);
-            g_free (mode);
-        }
+        mode = get_document_hint (ui);
+        g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
+                       TRUE, get_replace_pages (ui));
+        g_free (device);
+        g_free (mode);
     }
 }
 
@@ -390,7 +406,6 @@ rotate_left_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         return;
     page = book_view_get_selected (ui->priv->book_view);
     page_rotate_left (page);
-    ui->priv->default_orientation = page_get_orientation (page);
 }
 
 
@@ -405,7 +420,7 @@ rotate_right_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         return;
     page = book_view_get_selected (ui->priv->book_view);
     page_rotate_right (page);
-    ui->priv->default_orientation = page_get_orientation (page);
+    ui->priv->default_page_orientation = page_get_orientation (page);
 }
 
 
@@ -465,6 +480,14 @@ custom_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
         set_crop (ui, "custom");
 }
 
+void crop_toolbutton_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+crop_toolbutton_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    set_crop (ui, "custom");
+}
+
 
 void four_by_six_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
@@ -647,122 +670,6 @@ print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
-static void
-load_device_cache (SimpleScan *ui)
-{
-    gchar *filename;
-    GKeyFile *key_file;
-    gboolean result;
-    GError *error = NULL;
-    
-    filename = g_build_filename (g_get_user_cache_dir (), "simple-scan", "device_cache", NULL);
-    
-    key_file = g_key_file_new ();
-    result = g_key_file_load_from_file (key_file, filename, G_KEY_FILE_NONE, &error);
-    if (error) {
-        if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
-            g_warning ("Error loading device cache file: %s", error->message);
-        g_error_free (error);
-        error = NULL;
-    }
-    if (result) {
-        gchar **groups, **group_iter;
-
-        groups = g_key_file_get_groups (key_file, NULL);
-        for (group_iter = groups; *group_iter; group_iter++) {
-            gchar *label, *device;
-
-            label = *group_iter;
-            device = g_key_file_get_value (key_file, label, "device", &error);
-            if (error) {
-                g_warning ("Error getting device name for label '%s': %s", label, error->message);
-                g_error_free (error);
-                error = NULL;
-            }
-            
-            if (device)
-                ui_add_scan_device (ui, device, label);
-
-            g_free (device);
-        }
-
-        g_strfreev (groups);
-    }
-
-    g_free (filename);
-    g_key_file_free (key_file);
-}
-
-
-static void
-save_device_cache (SimpleScan *ui)
-{
-    GtkTreeModel *model;
-    GtkTreeIter iter;
-
-    g_debug ("Saving device cache");
-
-    model = ui->priv->device_model;
-    if (gtk_tree_model_get_iter_first (model, &iter)) {
-        GKeyFile *key_file;
-        gchar *data;
-        gsize data_length;
-        GError *error = NULL;
-
-        key_file = g_key_file_new ();
-        do {
-            gchar *name, *label;
-            gboolean detected;
-            
-            gtk_tree_model_get (model, &iter, 0, &name, 1, &label, 2, &detected, -1);
-            
-            if (detected) {
-                g_debug ("Storing device '%s' in cache", name);
-                g_key_file_set_value (key_file, label, "device", name);
-            }
-
-            g_free (name);
-            g_free (label);
-        } while (gtk_tree_model_iter_next (model, &iter));
-        
-        data = g_key_file_to_data (key_file, &data_length, &error);
-        if (data) {
-            gchar *dir, *filename;
-            GFile *file;
-            GFileOutputStream *stream;
-            GError *error = NULL;
-
-            dir = g_build_filename (g_get_user_cache_dir (), "simple-scan", NULL);
-            g_mkdir_with_parents (dir, 0700);
-            filename = g_build_filename (dir, "device_cache", NULL);
-
-            file = g_file_new_for_path (filename);
-            stream = g_file_replace (file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &error);
-            if (error) {
-                g_warning ("Error writing device cache: %s", error->message);
-                g_error_free (error);
-                error = NULL;
-            }
-            if (stream) {
-                g_output_stream_write_all (G_OUTPUT_STREAM (stream), data, data_length, NULL, NULL, &error);
-                if (error) {
-                    g_warning ("Error writing device cache: %s", error->message);
-                    g_error_free (error);
-                    error = NULL;
-                }
-                g_output_stream_close (G_OUTPUT_STREAM (stream), NULL, NULL);
-            }
-            g_free (data);
-
-            g_free (filename);
-            g_free (dir);        
-        }
-
-        g_key_file_free (key_file);
-    }
-}
-
-
 void about_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
@@ -811,8 +718,6 @@ quit (SimpleScan *ui)
     char *device, *document_type;
     gint i;
 
-    save_device_cache (ui);
-
     device = ui_get_selected_device (ui);
     if (device) {
         gconf_client_set_string(ui->priv->client, "/apps/simple-scan/selected_device", device, NULL);
@@ -830,9 +735,12 @@ quit (SimpleScan *ui)
     gconf_client_set_int(ui->priv->client, "/apps/simple-scan/window_height", ui->priv->window_height, NULL);
     gconf_client_set_bool(ui->priv->client, "/apps/simple-scan/window_is_maximized", ui->priv->window_is_maximized, NULL);
 
-    for (i = 0; orientation_keys[i].key != NULL && orientation_keys[i].orientation != ui->priv->default_orientation; i++);
+    for (i = 0; orientation_keys[i].key != NULL && orientation_keys[i].orientation != ui->priv->default_page_orientation; i++);
     if (orientation_keys[i].key != NULL)
         gconf_client_set_string(ui->priv->client, "/apps/simple-scan/scan_direction", orientation_keys[i].key, NULL);
+    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_width", ui->priv->default_page_width, NULL);
+    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_height", ui->priv->default_page_height, NULL);
+    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_dpi", ui->priv->default_page_dpi, NULL);
    
     g_signal_emit (G_OBJECT (ui), signals[QUIT], 0);
 }
@@ -882,6 +790,27 @@ window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
 }
 
 
+static void
+page_changed_cb (Page *page, SimpleScan *ui)
+{
+    ui->priv->default_page_width = page_get_scan_width (page);
+    ui->priv->default_page_height = page_get_scan_height (page);
+    ui->priv->default_page_dpi = page_get_dpi (page);
+    ui->priv->default_page_orientation = page_get_orientation (page);
+}
+
+
+static void
+page_added_cb (Book *book, Page *page, SimpleScan *ui)
+{
+    ui->priv->default_page_width = page_get_scan_width (page);
+    ui->priv->default_page_height = page_get_scan_height (page);
+    ui->priv->default_page_dpi = page_get_dpi (page);
+    ui->priv->default_page_orientation = page_get_orientation (page);
+    g_signal_connect (page, "image-changed", G_CALLBACK (page_changed_cb), ui);
+}
+
+
 static void
 page_removed_cb (Book *book, Page *page, SimpleScan *ui)
 {
@@ -952,8 +881,6 @@ ui_load (SimpleScan *ui)
     gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->mode_combo), renderer, "text", 1);
     gtk_combo_box_set_active (GTK_COMBO_BOX (ui->priv->mode_combo), 0);
 
-    /* Load previously detected scanners and select the last used one */
-    load_device_cache (ui);
     device = gconf_client_get_string (ui->priv->client, "/apps/simple-scan/selected_device", NULL);
     if (device) {
         GtkTreeIter iter;
@@ -979,14 +906,24 @@ ui_load (SimpleScan *ui)
                            ui->priv->preview_scroll,
                            GTK_WIDGET (gtk_builder_get_object (builder, "page_menu")));
 
-    /* Find default scan direction */
+    /* Find default page details */
     scan_direction = gconf_client_get_string(ui->priv->client, "/apps/simple-scan/scan_direction", NULL);
+    ui->priv->default_page_orientation = TOP_TO_BOTTOM;
     if (scan_direction) {
         gint i;
         for (i = 0; orientation_keys[i].key != NULL && strcmp (orientation_keys[i].key, scan_direction) != 0; i++);
         if (orientation_keys[i].key != NULL)
-            ui->priv->default_orientation = orientation_keys[i].orientation;
+            ui->priv->default_page_orientation = orientation_keys[i].orientation;
     }
+    ui->priv->default_page_width = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_width", NULL);
+    if (ui->priv->default_page_width <= 0)
+        ui->priv->default_page_width = 595;
+    ui->priv->default_page_height = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_height", NULL);
+    if (ui->priv->default_page_height <= 0)
+        ui->priv->default_page_height = 842;
+    ui->priv->default_page_dpi = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_dpi", NULL);
+    if (ui->priv->default_page_dpi <= 0)
+        ui->priv->default_page_dpi = 72;
 
     /* Restore window size */
     ui->priv->window_width = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/window_width", NULL);
@@ -1183,6 +1120,7 @@ ui_init (SimpleScan *ui)
 
     ui->priv->book = book_new ();
     g_signal_connect (ui->priv->book, "page-removed", G_CALLBACK (page_removed_cb), ui);
+    g_signal_connect (ui->priv->book, "page-added", G_CALLBACK (page_added_cb), ui);
     g_signal_connect (ui->priv->book, "cleared", G_CALLBACK (book_cleared_cb), ui);
    
     ui->priv->client = gconf_client_get_default();
